// StageController.js
// Version: 0.0.1
// Event: Lens Initialized
// Description: Controls and calls functions in all stage scripts

// @input Component.ScriptComponent[] stageScripts
// @input bool loopStages {"hint" : "Allows repeat after all stages are completed"}
// @input bool usePersistentStorage

//@input Component.Text debug

var currentStageIndex = 0;

//persistent storage
var store = global.persistentStorageSystem.store; 
var stageKey = "currentStage";

initializeAllStages();

function initializeAllStages() {
    for (var i = 0; i < script.stageScripts.length; i++) {
        if (script.stageScripts[i]) {
            script.stageScripts[i].api.initialize(onStageCompleted);
        }
    }
    onStageStart();
}

function onStageStart() {
    if (script.usePersistentStorage) {
        currentStageIndex = store.getInt(stageKey);
    }

    script.debug.text = "On Start " + currentStageIndex.toString();
    
    if (script.stageScripts[currentStageIndex].api.onStageStarted) {
        script.stageScripts[currentStageIndex].api.onStageStarted();
    }
}

function onStageCompleted() {
    currentStageIndex += 1;
    script.debug.text = "On Complete " + currentStageIndex.toString();
    
    if (currentStageIndex >= script.stageScripts.length) {
        if (script.usePersistentStorage) {        
            store.putInt(stageKey, 0);
        }
        if (script.loopStages) {
            debugPrint("LoopStages");
            currentStageIndex = 0;
            initializeAllStages();
        }else{
            return;
        }
    }else{
        if (script.usePersistentStorage) {
            store.putInt(stageKey, currentStageIndex);
        }
        onStageStart();
    }

}

function debugPrint(message) {
    print("StageController: " + message);
}

global.stageController = {};
