// StageScript.js
// Version: 0.0.1
// Event: Lens Initialized
// Description: Manages all things related to current stage in the game


//----------stage related----------
//scanMarker, faceEvent,distanceCheck, touchEvent, customTrigger, multipleTriggers
// @input string triggerType = "scanMarker" {"widget" : "combobox", "values" : [{"label" : "Scan Marker", "value" : "scanMarker"}, {"label" : "Face Event", "value" : "faceEvent"},{"label" : "Distance Check", "value" : "distanceCheck"}, {"label" : "Touch Event", "value" : "touchEvent"}, {"label" : "Custom Trigger", "value" : "customTrigger"}, {"label" : "Sub Stage Triggers", "value" : "multipleTriggers"}]}
// @input float triggerDelay = 0 {"hint":"delay time after object has been successfully triggered"}
//----------scan marker----------
//@input Component.MarkerTrackingComponent marker {"showIf":"triggerType","showIfValue":"scanMarker"}
//----------face event----------
//@input string faceEventType = "MouthOpenedEvent" {"showIf": "triggerType", "showIfValue": "faceEvent", "values": [{"value": "MouthOpenedEvent", "label": "Mouth Opened"}, {"value": "MouthClosedEvent", "label": "Mouth Closed"}, {"value": "BrowsRaisedEvent", "label": "Brows Raised"}, {"value": "BrowsLoweredEvent", "label": "Brows Lowered"}, {"value": "BrowsReturnedToNormalEvent", "label": "Brows Returned to Normal"}, {"value": "FaceFoundEvent", "label": "Face Found"}, {"value": "FaceLostEvent", "label": "Face Lost"}, {"value": "KissStartedEvent", "label": "Kiss Started"}, {"value": "KissFinishedEvent", "label": "Kiss Finished"}, {"value": "SmileStartedEvent", "label": "Smile Started"}, {"value": "SmileFinishedEvent", "label": "Smile Finished"}], "widget": "combobox"}
//@input int faceIndex = 0 {"showIf": "triggerType", "showIfValue": "faceEvent"}
//----------distance check----------
//@input SceneObject distanceCheckObjectA {"showIf": "triggerType", "showIfValue": "distanceCheck", "label": "Object A"}
//@input SceneObject distanceCheckObjectB {"showIf": "triggerType", "showIfValue": "distanceCheck", "label": "Object B"}
//@input int distanceCheckCompareType = -1 {"showIf": "triggerType", "showIfValue": "distanceCheck", "values": [{"value": -1, "label": "Is Less Than"}, {"value": 0, "label": "Is Equal To"}, {"value": 1, "label": "Is Greater Than"}], "widget": "combobox", "label": "Compare Type"}
//@input float distanceCheckThreshold = 1 {"showIf": "triggerType", "showIfValue": "distanceCheck", "label": "Threshold"}
/*Component.Text debuggerText {"showIf": "triggerType", "showIfValue": "distanceCheck", "label": "Threshold"}
*/
//----------touch event----------
//@input string touchEventType = "TapEvent" {"showIf": "triggerType", "showIfValue": "touchEvent", "values": [{"value": "TapEvent", "label": "Tap"}, {"value": "TouchStartEvent", "label": "Touch Start"}, {"value": "TouchMoveEvent", "label": "Touch Move"}, {"value": "TouchEndEvent", "label": "Touch End"}], "widget": "combobox", "label": "Event Type"}
//@input Component.BaseMeshVisual touchObject {"showIf": "triggerType", "showIfValue": "touchEvent"}
//----------custom trigger----------
//@input string eventTriggerName {"showIf": "triggerType", "showIfValue": "customTrigger","label" : "Trigger Name"}
//----------sub stage triggers----------
//@input Component.ScriptComponent[] subStageScripts {"label": "Sub Stages", "showIf":"triggerType","showIfValue":"multipleTriggers"}
//@input bool hasOrder {"showIf":"triggerType","showIfValue":"multipleTriggers"}
//--------------------------------
//@ui {"widget":"separator"}
//@input bool isSubScript
//@input Component.ScriptComponent parentScript {"showIf":"isSubScript","showIfValue":"true"}

//----------stage elements------------
//@ui {"widget":"separator"}
//@ui {"widget":"label", "label":"On Stage Started"}
//@input SceneObject[] objectsEnabled {"label" : "Enable SceneObjects On Start"}
//@input SceneObject[] objectsDisabled {"label" : "Disable SceneObjects On Start"}
//@input Component.ScriptComponent[] apiToCall {"label" : "API to Call On Start"}
//@input string[] behaviorTriggerToCall {"label" : "Send Behavior Triggers On Start"}

//----------response type----------
//@ui {"widget":"separator"}
//@ui {"widget":"label", "label":"On Stage Completed"}

//effect length
//use behavior trigger chain
//use external api (particle burst, check out the UI widget project)
//@input bool hasResponse
//@input SceneObject[] objectsEnabledOnComplete {"showIf":"hasResponse","showIfValue":"true","label":"Obj To Enable On Complete"}
//@input SceneObject[] objectsDisabledOnComplete {"showIf":"hasResponse","showIfValue":"true","label":"Obj To Disable On Complete"}
//@input Component.ScriptComponent[] apiToCallOnComplete {"showIf":"hasResponse","showIfValue":"true"}
//@input string[] behaviorTriggerOnComplete {"showIf":"hasResponse","showIfValue":"true", "label" : "Send Behavior Triggers On Complete"}

//----------hint-----------
//@ui {"widget":"separator"}

//@input bool hasHint
//hint object with recursive fade tween set
//@input SceneObject hintObject {"showIf":"hasHint","showIfValue":"true","hint" : "hintObject must have TweenAlpha scripts attached with the according name for showTween & hideTween"}
//@input string showTween = "FADE_IN" {"showIf":"hasHint","showIfValue":"true"}
//@input string hideTween = "FADE_OUT" {"showIf":"hasHint","showIfValue":"true"}

//----------debug-----------
//@ui {"widget":"separator"}
//@ui {"widget":"label", "label":"check this if you want this stage to be automatically completed"}
//@input bool automaticPass


//not ready, initialized, started, completed
var Stage = {NOTREADY: -1, INITIALIZED: 0, STARTED: 1, FINISHED: 2};
var stageStatus = Stage.NOTREADY;



script.api.initialize = function(stageControllerCompleted) {

    for (var i=0;i<script.objectsEnabled.length;i++) {
        script.objectsEnabled[i].enabled = false;
    }
    for (var i=0;i<script.objectsEnabled.length;i++) {
        script.objectsEnabled[i].enabled = false;
    }
    
    if (script.hasHint) {
        script.hintObject.enabled = false;
    }
    
    if (script.triggerType == "multipleTriggers") {
        for (var i = 0; i < script.subStageScripts.length; i++) {
            if (script.subStageScripts[i]) {
                script.subStageScripts[i].api.initialize();
            }
        }
    }
    
    onStageControllerCompleted = wrapFunction(stageControllerCompleted);
    
    stageStatus = Stage.INITIALIZED;
};

script.api.checkStatus = function() {
    return stageStatus;
};

//try {
//  onStageStarted();
//} catch (error) {
//  print(error);
//}


function onStageStarted() {
    
    switch (script.triggerType) {
        case "scanMarker":
            script.marker.onMarkerFound = wrapFunction(script.marker.onMarkerFound, onStageFinished);
            break;
        case "faceEvent":
            var faceEvent = script.createEvent(script.faceEventType);
            faceEvent.faceIndex = script.faceIndex;
            faceEvent.bind(onStageFinished);
            break;
        case "distanceCheck":
            break;
        case "touchEvent":
            var targetScript = script;
            if (script.touchObject) {
                var targetObj = script.touchObject.getSceneObject();
                var touchComponent = getOrAddComponent(targetObj, "Component.TouchComponent");
                touchComponent.addMeshVisual(script.touchObject);
                targetScript = targetObj.createComponent("Component.ScriptComponent");
            }
            targetScript.createEvent(script.touchEventType).bind(onStageFinished);        
            break;
        case "customTrigger":
            global.behaviorSystem.addCustomTriggerResponse(script.eventTriggerName, onStageFinished);
            break;
        case "multipleTriggers":
            for (var i = 0; i < script.subStageScripts.length; i++) {
                if (script.subStageScripts[i]) {
                    script.subStageScripts[i].api.onStageStarted();
                }
            }
            break;
    }
    
    setStageStart();
    
    stageStatus = Stage.STARTED;
    
    if (script.automaticPass) {
        onStageFinished();
        debugPrint("manually passing stage");
    }
   
}



function onStageFinished() {
    if (stageStatus == Stage.STARTED) {
        if(script.triggerDelay > 0){
            onStageFinishedDelayedEvent.reset(script.triggerDelay);
        }else{
            onStageFinishedFunc();
        }
    }
}

var onStageFinishedDelayedEvent = script.createEvent("DelayedCallbackEvent");
onStageFinishedDelayedEvent.bind(function(eventData){
    onStageFinishedFunc();
});

function onStageFinishedFunc(){
    if (script.triggerType == "multipleTriggers") {
        if (isMultipleStagesFinished()) {
            stageStatus = Stage.FINISHED;
        } else if (script.automaticPass) {
            stageStatus = Stage.FINISHED;
        } else {
            return;
        }
    } else {
        stageStatus = Stage.FINISHED;
    }
    
    if (stageStatus != Stage.FINISHED) {
        return;
    }
    
    setResponses();
    
    if (script.isSubScript) {
        script.parentScript.api.onStageFinished();
    } else {
        onStageControllerCompleted();
    }
    
    if (script.hasHint) {
        global.tweenManager.startTween(script.hintObject, script.hideTween);
    }
}


function onStageControllerCompleted() {}

function setStageStart() {
    if (script.hasHint) {
        script.hintObject.enabled = true;
        global.tweenManager.startTween(script.hintObject, script.showTween);
    }
    
    for (var i=0;i<script.objectsEnabled.length;i++) {
        script.objectsEnabled[i].enabled = true;
    }
    for (var i=0;i<script.objectsDisabled.length;i++) {
        script.objectsDisabled[i].enabled = false;
    }
    for (var i=0;i<script.apiToCall.length;i++) {
        script.apiToCall[i].api["onResponse"]();
    }
    for (var i=0;i<script.behaviorTriggerToCall.length;i++) {
        global.behaviorSystem.sendCustomTrigger(script.behaviorTriggerToCall[i]);
    }
}

function setResponses() {

    if (script.hasHint) {
        global.tweenManager.startTween(script.hintObject, script.hideTween);
    }
    
    for (var i = 0; i < script.objectsEnabledOnComplete.length; i++) {
        if (script.objectsEnabledOnComplete[i]) {
            script.objectsEnabledOnComplete[i].enabled = true;
        }
    }
    
    for (var i = 0; i < script.objectsDisabledOnComplete.length; i++) {
        if (script.objectsDisabledOnComplete[i]) {
            script.objectsDisabledOnComplete[i].enabled = false;
        }
    }
    
    for (var i = 0; i < script.apiToCallOnComplete.length; i++) {
        script.apiToCallOnComplete[i].api["onResponse"]();
    }
    
    for (var i = 0; i < script.behaviorTriggerOnComplete.length; i++) {
        if (script.behaviorTriggerOnComplete[i]) {
            global.behaviorSystem.sendCustomTrigger(script.behaviorTriggerOnComplete[i]);
        }
    }    
    
    if (script.triggerType == "multipleTriggers") {
        for (i=0;i<script.subStageScripts.length;i++) {
            script.subStageScripts[i].api.setResponses();
        }
        
    }
    
}


function isMultipleStagesFinished() {
    var stageFinished = 0;
    for (var i = 0; i < script.subStageScripts.length; i++) {
        if (script.subStageScripts[i].api.getStageStatus() == Stage.FINISHED) {
            stageFinished ++;
        }
    }
    //debugPrint("check if multiple stage is passed " + stageCompleted)
    return (stageFinished == script.subStageScripts.length);
}

var wasOppositeDistance = false;

function onUpdate() {
    if (stageStatus == Stage.STARTED) {
        if (script.triggerType == "distanceCheck") {
            if (isMatchingDistance(script.distanceCheckObjectA, script.distanceCheckObjectB, script.distanceCheckThreshold, script.distanceCheckCompareType)) {
                if (wasOppositeDistance) {
                    onStageFinished();
                }
            } else {
                wasOppositeDistance = true;
            }
        }
        
    }
    
    
}
script.createEvent("UpdateEvent").bind(onUpdate);

function isMatchingDistance(objA, objB, threshold, relation) {
    var pass = false;
    var distance = objA.getTransform().getWorldPosition().distance(objB.getTransform().getWorldPosition());
    //script.debuggerText.text = distance.toString();    
    switch (relation) {
        case -1:
            pass = (distance < threshold) ? true : false;
            break;
        case 0:
            pass = (distance == threshold) ? true : false;
            break;
        case 1:
            pass = (distance > threshold) ? true : false;
            break;
    }
    return pass;
}

function getOrAddComponent(obj, componentType) {
    return obj.getComponent(componentType) || obj.createComponent(componentType);
}

function getStageStatus() {
    return stageStatus;
}

function wrapFunction(orgFunc, newFunc) {
    if (!orgFunc) {
        return newFunc;
    }
    if (!newFunc) {
        return orgFunc;
    }
    return function() {
        orgFunc();
        newFunc();
    };
}

function debugPrint(message) {
    print("StageScript: " + script.getSceneObject().name + ", " + message);
}


script.api.onStageStarted = onStageStarted;
script.api.onStageFinished = onStageFinished;
script.api.getStageStatus = getStageStatus;
script.api.setResponses = setResponses;